"""Interact with the filesystem

All functions that interact with the filesystem are located in this subpackage.

Modules:
    io_ascii: Functions to interact with ASCII files.
    io_yaml: Functions to interact with YAML files.
    classic_swap: Functions to load with classic SWAP input files.
"""
