"""GIS functionality for pyswap.

Currently this module provides limited functionality with respect to GIS data.
However, it is intended to be expanded in the future.

Modules:
    location: Location class and related functionality.
"""

from pyswap.gis.location import Location

__all__ = ["Location"]
